%define major		12
%define libname		%mklibname %{name} %{major}

%define mallocmajor	2
%define malloclib	%mklibname %{name}malloc %{mallocmajor}
%define mallocproxylib	%mklibname %{name}malloc-proxy %{mallocmajor}

%define bindmajor	3
%define bindlib		%mklibname %{name}bind2.5_ %{bindmajor}

%define irmlmajor	1
%define irmllib		%mklibname irml %{irmlmajor}

%define devname		%mklibname %{name} -d

Summary:	Library to abstract low-level threading details
Name:		tbb
Version:	2021.13.0
Release:	%mkrel 1
License:	Apache-2.0 AND BSD-3-Clause
Group:		Development/C++
URL:		https://threadingbuildingblocks.org/

Source0:	https://github.com/oneapi-src/oneTBB/archive/v%{version}/%{name}-%{version}.tar.gz
# These two are downstream sources.
Source7:	tbbmalloc.pc
Source8:	tbbmalloc_proxy.pc

BuildRequires:	cmake
BuildRequires:	doxygen
BuildRequires:	gcc-c++
BuildRequires:	hwloc
BuildRequires:	hwloc-devel
BuildRequires:	libgomp-devel
BuildRequires:	python3-devel
BuildRequires:	%{py3_dist setuptools}
BuildRequires:	%{py3_dist sphinx}
BuildRequires:	%{py3_dist sphinx-rtd-theme}
BuildRequires:	swig

%description
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.

TBB requires fewer lines of code to achieve parallelism than other
threading models.  The applications you write are portable across
platforms.  Since the library is also inherently scalable, no code
maintenance is required as more processor cores become available.


%package -n %{libname}
Summary:	Library to abstract low-level threading details
Group:		System/Libraries

%description -n %{libname}
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.

TBB requires fewer lines of code to achieve parallelism than other
threading models.  The applications you write are portable across
platforms.  Since the library is also inherently scalable, no code
maintenance is required as more processor cores become available.


%package -n %{malloclib}
Summary:	Library to abstract low-level threading details
Group:		System/Libraries

Conflicts:	%{_lib}tbb2 < 2020.3-5

%description -n %{malloclib}
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.


%package -n %{mallocproxylib}
Summary:	Library to abstract low-level threading details
Group:		System/Libraries

Conflicts:	%{_lib}tbb2 < 2020.3-5

%description -n %{mallocproxylib}
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.


%package -n %{bindlib}
Summary:	Threading Building Blocks (TBB) NUMA support library
Group:		System/Libraries

%description -n %{bindlib}
Threading Building Blocks (TBB) NUMA support library.


%package -n %{irmllib}
Summary:	Library to abstract low-level threading details
Group:		System/Libraries

%description -n %{irmllib}
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.

TBB requires fewer lines of code to achieve parallelism than other
threading models.  The applications you write are portable across
platforms.  Since the library is also inherently scalable, no code
maintenance is required as more processor cores become available.


%package -n %{devname}
Summary:	The Threading Building Blocks C++ headers and shared development libraries
Group:		Development/C++
Requires:	%{libname} = %{version}-%{release}
Requires:	%{malloclib} = %{version}-%{release}
Requires:	%{mallocproxylib} = %{version}-%{release}
Requires:	%{bindlib} = %{version}-%{release}
Requires:	%{irmllib} = %{version}-%{release}
Provides:	%{name}-devel = %{version}-%{release}

%description -n %{devname}
Header files and shared object symlinks for the Threading Building
Blocks (TBB) C++ libraries.


%package doc
Summary:	The Threading Building Blocks documentation
Group:		Documentation

%description doc
PDF documentation for the user of the Threading Building Block (TBB)
C++ library.


%package -n python3-%{name}
Summary:	Python 3 TBB module
Group:		Development/Python
%{?python_provide:%python_provide python3-%{name}}

%description -n python3-%{name}
Python 3 TBB module.


%prep
%autosetup -p1 -n oneTBB-%{version}

# Invoke the right python binary directly
for fil in $(grep -Frl %{_bindir}/env python); do
    sed -i.orig 's,env python3,python3,' $fil
    touch -r $fil.orig $fil
    rm $fil.orig
done

# Use local objects.inv for intersphinx
sed -e "s|\('https://docs\.python\.org/': \)None|\1'%{_docdir}/python3-docs/html/objects.inv'|" \
    -i doc/GSG/conf.py doc/main/conf.py

%generate_buildrequires
cd python
%pyproject_buildrequires

%build
export TBBROOT=$PWD
export PYTHONPATH=$(sed "s,%{_prefix},$PWD/%{_vpath_builddir}/python/build," <<< %{python3_sitearch})
%cmake \
    -DCMAKE_CXX_STANDARD=17 \
    -DTBB4PY_BUILD:BOOL=ON \
    -DTBB_STRICT:BOOL=OFF
%cmake_build

# The python package is not built the Fedora way.  Do it over.
unset PYTHONPATH
export LD_LIBRARY_PATH=$(ls -1d $PWD/%{_vpath_builddir}/*relwithdebinfo)
export LDFLAGS="-L $LD_LIBRARY_PATH %{build_ldflags}"
cd python
%pyproject_wheel
cd -

# Build documentation
export BUILD_TYPE=oneapi
sphinx-build doc/GSG getting-started
sphinx-build doc/main html

%install
%cmake_install

# The python package is not installed the Fedora way.  Do it over.
rm -fr %{buildroot}%{python3_sitearch}
cd python
%pyproject_install
cd -

mkdir -p %{buildroot}%{_libdir}/pkgconfig
for file in %{SOURCE7} %{SOURCE8}; do
    target=%{buildroot}%{_libdir}/pkgconfig/$(basename ${file})
    sed 's/_FEDORA_VERSION/%{version}/' $file > $target
    touch -r $file $target
done

# Upstream installs tbb32.pc on 32-bit but it's already in a separate directory
# because %%_libdir is different for 32-bit and 64-bit, so rename it to tbb.pc.
if [ -f %{buildroot}%{_libdir}/pkgconfig/%{name}32.pc ]; then
    mv %{buildroot}%{_libdir}/pkgconfig/%{name}32.pc %{buildroot}%{_libdir}/pkgconfig/%{name}.pc
fi

rm -fr %{buildroot}%{_datadir}/doc

%check
# Running the tests in parallel often leads to resource exhaustion.
ctest --output-on-failure --force-new-ctest-process

%files -n %{libname}
%doc RELEASE_NOTES.md README.md
%license LICENSE.txt
%{_libdir}/libtbb.so.%{major}{,.*}

%files -n %{malloclib}
%license LICENSE.txt
%{_libdir}/libtbbmalloc.so.%{mallocmajor}{,.*}

%files -n %{mallocproxylib}
%license LICENSE.txt
%{_libdir}/libtbbmalloc_proxy.so.%{mallocmajor}{,.*}

%files -n %{bindlib}
%license LICENSE.txt
%{_libdir}/libtbbbind_2_5.so.%{bindmajor}{,.*}

%files -n %{irmllib}
%license LICENSE.txt
%{_libdir}/libirml.so.%{irmlmajor}

%files -n %{devname}
%doc cmake/README.md
%{_includedir}/oneapi/
%{_includedir}/tbb/
%{_libdir}/cmake/TBB/
%{_libdir}/lib*.so
%{_libdir}/pkgconfig/*.pc

%files doc
%doc getting-started
%doc html

%files -n python3-%{name}
%doc python/README.md
%{python3_sitearch}/TBB*
%{python3_sitearch}/tbb/
%{python3_sitearch}/__pycache__/TBB*


%changelog
* Wed Jul 10 2024 daviddavid <daviddavid> 2021.13.0-1.mga10
+ Revision: 2080559
- new version: 2021.13.0

* Wed Apr 24 2024 daviddavid <daviddavid> 2021.12.0-1.mga10
+ Revision: 2060830
- new version: 2021.12.0

* Sun Jan 28 2024 daviddavid <daviddavid> 2021.11.0-1.mga10
+ Revision: 2035668
- new version: 2021.11.0
- switch to cmake build system
- split out in their own sub-pkgs all libraries
- new tbbbind sub-pkg for the NUMA support library
+ papoteur <papoteur>
- Mass Rebuild - Python 3.12

* Sat Mar 26 2022 umeabot <umeabot> 2020.3-4.mga9
+ Revision: 1827818
- Mageia 9 Mass Rebuild

* Mon Feb 14 2022 tmb <tmb> 2020.3-3.mga9
+ Revision: 1774922
- rebuild for python 3.10

* Tue Mar 23 2021 daviddavid <daviddavid> 2020.3-2.mga9
+ Revision: 1708116
- rebuild for python 3.9

* Tue Sep 01 2020 daviddavid <daviddavid> 2020.3-1.mga8
+ Revision: 1620589
- new version: 2020.3 (sync with Fedora)

* Sun Feb 16 2020 umeabot <umeabot> 2019.2-5.mga8
+ Revision: 1533961
- Mageia 8 Mass Rebuild

* Wed Sep 25 2019 daviddavid <daviddavid> 2019.2-4.mga8
+ Revision: 1445902
- rebuild for python3.8

* Tue Jan 08 2019 tv <tv> 2019.2-3.mga7
+ Revision: 1351598
- rebuild with python 3.7

* Tue Dec 04 2018 wally <wally> 2019.2-2.mga7
+ Revision: 1338320
- new version 2019 update 2 (sync with Fedora)
- drop special SSE2 build for 32-bit x86 as that is now default
- drop python2 support
+ umeabot <umeabot>
- Mageia 7 Mass Rebuild

* Mon Aug 06 2018 wally <wally> 2018.5-1.mga7
+ Revision: 1248345
- new version 2018 Update 5
- update license to ASL 2.0
- sync with Fedora

* Wed Jan 13 2016 pterjan <pterjan> 4.3-1.20140724.5.mga6
+ Revision: 922489
- Steal a patch from Debian to always use GCC atomics

* Sat Aug 22 2015 akien <akien> 4.3-1.20140724.4.mga6
+ Revision: 868028
- Rebuild for gcc 5.2.1

* Wed Oct 15 2014 umeabot <umeabot> 4.3-1.20140724.3.mga6
+ Revision: 741515
- Second Mageia 5 Mass Rebuild

* Tue Sep 16 2014 umeabot <umeabot> 4.3-1.20140724.2.mga5
+ Revision: 689731
- Mageia 5 Mass Rebuild

* Wed Aug 27 2014 akien <akien> 4.3-1.20140724.1.mga5
+ Revision: 668602
- Version 4.3.0

* Sun Jun 15 2014 wally <wally> 4.1-1.20121003.1.mga5
+ Revision: 636629
- fix libification

* Thu Dec 13 2012 shlomif <shlomif> 4.1-1.20121003.mga3
+ Revision: 330401
- New version 4.1-something

* Wed Nov 14 2012 shlomif <shlomif> 4.0-5.20120408.mga3
+ Revision: 317951
- Correct the groups.
- Change to mkrel
- tbb package based on the Fedora package.


* Sat Jul 21 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.0-5.20120408
- Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild

* Thu Jun  7 2012 Petr Machata <pmachata@redhat.com> - 4.0-4.20120408
- Rebase to 4.0 update 4
- Refresh Getting_Started.pdf, Reference.pdf, Tutorial.pdf
- Provide pkg-config files
- Resolves: #825402

* Thu Apr 05 2012 Karsten Hopp <karsten@redhat.com> 4.0-3.20110809
- tbb builds now on PPC(64)

* Sat Jan 14 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.0-2.20110809
- Rebuilt for https://fedoraproject.org/wiki/Fedora_17_Mass_Rebuild

* Tue Oct 18 2011 Petr Machata <pmachata@redhat.com> - 4.0-1.20110809
- Rebase to 4.0
  - Port the mfence patch
  - Refresh the documentation bundle

* Tue Jul 26 2011 Petr Machata <pmachata@redhat.com> - 3.0-1.20110419
- Rebase to 3.0-r6
  - Port both patches
  - Package Design_Patterns.pdf
  - Thanks to Richard Shaw for initial rebase patch
- Resolves: #723043

* Wed Feb 09 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.2-3.20090809
- Rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild

* Thu Jun 10 2010 Petr Machata <pmachata@redhat.com> - 2.2-2.20090809
- Replace mfence instruction with xchg to make it run on ia32-class
  machines without SSE2.
- Resolves: #600654

* Tue Nov  3 2009 Petr Machata <pmachata@redhat.com> - 2.2-1.20090809
- New upstream 2.2
- Resolves: #521571

* Sun Jul 26 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.1-3.20080605
- Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild

* Wed Feb 25 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.1-2.20080605
- Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild

* Fri Jun 13 2008 Petr Machata <pmachata@redhat.com> - 2.1-1.20080605
- New upstream 2.1
  - Drop soname patch, parallel make patch, and GCC 4.3 patch

* Wed Feb 13 2008 Petr Machata <pmachata@redhat.com> - 2.0-4.20070927
- Review fixes
  - Use updated URL
  - More timestamp preservation
- Initial import into Fedora CVS

* Mon Feb 11 2008 Petr Machata <pmachata@redhat.com> - 2.0-3.20070927
- Review fixes
  - Preserve timestamp of installed files
  - Fix soname not to contain "debug"

* Tue Feb  5 2008 Petr Machata <pmachata@redhat.com> - 2.0-2.20070927
- Review fixes
  - GCC 4.3 patchset
  - Add BR util-linux net-tools
  - Add full URL to Source0
  - Build in debug mode to work around problems with GCC 4.3

* Mon Dec 17 2007 Petr Machata <pmachata@redhat.com> - 2.0-1.20070927
- Initial package.
- Using SONAME patch from Debian.
